/*
   This file defines a simple data structure that holds
   information that describes the dimensions of a box.

   The data structure now comes with a constructor that
   is used to "construct" the data structure with the
   appropriate initial values.

   A Box4 object knows how to compute its volume.

   A Box4 object knows how to return a printable
   description of itself.

   A Box4 object doe not allow its dimensions to be negative
   numbers.

   A Box4 object holds data that is private, so the data can
   only be manipulated through methods that belong to the object.

   A Box4 object is "immutable", that is, once a Box4 object
   is created, it cannot be changed.

   A Box4 object knows how to compute its "length plus girth"
   (which is used by shipping companies) where "length" means
   the largest of the three dimensions.
*/

public class Box4
{
   // three (private) instance variables
   private double length;
   private double width;
   private double height;

   // define a constructor
   public Box4(double length, double width, double height)
   {
      if (length <= 0 || width <= 0 || height <= 0)
         throw new IllegalArgumentException("Illegal dimension");

      this.length = length;
      this.width  = width;
      this.height = height;
   }//Box4 constructor


   // define three "get methods",
   // one for each field of the class
   public double getLength()
   {
      return length;
   }

   public double getWidth()
   {
      return width;
   }

   public double getHeight()
   {
      return height;
   }

   // define a method for computing the volume of the box
   public double volume()
   {
      return this.height * this.width * this.length;
   }

   // define a method to compute "length plus girth"
   // (which is used by shipping companies)
   public double lengthPlusGirth()
   {
      // length should be the largest
      // dimension of the box
      double length = this.length;
      double width = this.width;
      double height = this.height;
      if (width > length)
      {
         // if width is larger than length
         // then swap the two values
         double temp = length;
         length = width;
         width = temp;
      }
      if (height > length)
      {
         // if height is larger than length
         // then swap the two values
         double temp = length;
         length = height;
         height = temp;
      }
      return length + 2*width + 2*height;
   }


   // define a toString method
   public String toString()
   {
       return "The dimensions of the box are\n"
               + "height is " + height + ",\n"
               + "width is "  + width  + ",\n"
               + "length is " + length + ".";
   }
}
