/*
   This program makes use of the OrderItem class and its subclasses.

   In this example, some of the subclasses of OrderItem might implement
   the Returnable interface. A test using the instanceof operator
   determines which entries from the array order[] are Returnable.

   Notice that the toString() method is called polymorphically
   on the entries from the array order[]. If it were not for
   polymorphism, the toString() method called would be the one
   from the OrderItem class. But instead, it is the toString()
   method from the class of the object referred to (Glove, HatA,
   or HatB) that is called.
*/

public class UseOrderItem
{
   public static void main(String[] args)
   {
      // Create an array of type OrderItem.
      OrderItem[] order = new OrderItem[4];

      // Fill the array with objects of type Glove, HatA or HatB.
      order[0] = new Glove(3, 12.95, Glove.MEDIUM, Glove.WOMEN, Glove.BLACK);
      order[1] = new Glove(1, 14.95, Glove.LARGE, Glove.MEN, Glove.BROWN);
      order[2] = new HatA(1, 9.95, HatA.LARGE, HatA.BASEBALL, HatA.RED);
      order[3] = new HatB(1, 9.95, HatB.SMALL, HatB.BEANIE, HatB.BLUE);

      // Print out the items from this order.
      for (int i = 0; i < order.length; i++)
      {
         System.out.println( order[i] );
         // Check if the item is returnable and if so, print a message.
         // Notice that we use a cast in order to access the
         // behavior from the Returnable interface. Why is this?
         if ( order[i] instanceof Returnable )
         {
            System.out.println("This item is returnable within "
                               + ((Returnable)order[i]).withinNumberOfDays()
                               + " days.");
         }
         // output a blank line
         System.out.println();
      }
   }
}
